package org.apache.poi.ss.usermodel;

import java.util.EnumSet;
import java.util.Set;

/**
 * Error/warning messages to turn off for certain cells. Excel will normally
 * display these by marking the cell with a green triangle in the upper left
 * corner of the cell.
 *
 * <p>
 * Several types of warnings can be turned off for the same range of cells.
 */
public interface IgnoredError {

    /**
     * Error/warning types that can be ignored for a cell or range of cells.
     *
     * <p>
     * Note that not <i>all</i> of these may be supported in older Excel
     * formats.
     */
    public enum Type {

        /**
         * Ignore calculation errors.
         */
        EVAL_ERROR("evalError"),
        /**
         * Ignore errors arising from the formatting of date/time values.
         */
        TWO_DIGIT_TEXT_YEAR("twoDigitTextYear"),
        /**
         * Ignore errors arising from the formatting of numeric values.
         */
        NUMBER_STORED_AS_TEXT("numberStoredAsText"),
        /**
         * Ignore errors arising from formulas that are inconsistent with
         * formulas in neighboring cells.
         */
        FORMULA("formula"),
        /**
         * Ignore errors arising from formulas that contain references to less
         * than the entirety of a range containing contiguous data.
         */
        FORMULA_RANGE("formulaRange"),
        /**
         * Ignore errors arising from unprotected formulas.
         */
        UNLOCKED_FORMULA("unlockedFormula"),
        /**
         * Ignore errors arising from references to empty cells.
         */
        EMPTY_CELL_REFERENCE("emptyCellReference"),
        /**
         * Ignore errors arising from data validation.
         */
        LIST_DATA_VALIDATION("listDataValidation"),
        /**
         * Ignore errors arising from calculated column formulas.
         */
        CALCULATED_COLUMN("calculatedColumn");

        private final String xmlAttribName;

        Type(String xmlAttribName) {
            this.xmlAttribName = xmlAttribName;
        }

        /**
         * Gets the corresponding OOXML attribute name.
         *
         * @return
         */
        public String getXmlAttribName() {
            return xmlAttribName;
        }

    }

    /**
     * Gets the error/warning types which applies to the cells in
     * {@link #getCellRanges()}.
     *
     * @return
     */
    public EnumSet<Type> getIgnoredErrorTypes();

    /**
     * Get the cell ranges for which the error/warning types from {@link #getIgnoredErrorTypes()
     * } are ignored.
     *
     * @return
     */
    public Set<CellRange> getCellRanges();

    /**
     * Implementers must implement this as well as {@link #hashCode()}. Two
     * objects are considered equal if they have the same EnumSet of types and
     * the same content in the Set of CellRange.
     *
     * @param obj
     * @return
     */
    @Override
    public boolean equals(Object obj);

    @Override
    public int hashCode();
}
